@@ -1,3 +1,4 @@
+from __future__ import print_function
 from activedirectory import Client, Creds, activate
 
 domain = 'freeadi.org'
@@ -12,4 +13,4 @@ client = Client(domain)
 users = client.search('(objectClass=user)')
 for dn,attrs in users:
     name = attrs['sAMAccountName'][0]
-    print '-> %s' % name
+    print('-> %s' % name)
